import telebot
from telebot.types import InlineKeyboardMarkup, InlineKeyboardButton

### LOAD TOKEN ###
import json, os

if not os.path.exists('db.json'):
	db = {"token": "None"}
	js = json.dumps(db, indent=2)
	with open("db.json", "w") as outfile:
		outfile.write(js)

	print('Input token in "None" (db.json)')
	exit()
with open('db.json', 'r') as openfile:
	db = json.load(openfile)

API_TOKEN = db["token"]
bot = telebot.TeleBot(API_TOKEN)
##################


@bot.message_handler(commands=['help', 'start'])
def send_welcome(message):
	bot.reply_to(message, f"""Это подобие ChatGPT на минималках.

__ Есть 3 версии:
0.1 - Простейшая, быстрейшая, краткая, не помнит что вы говорили.
0.2 - Умнее, относительно быстрая, помнит что вы говорили.
3.5 - Самая умная, есть цензура, помнит что вы говорили.
/m - Выбор модели

__ Список команд:
help - Справка
i - Информация о конфигурации
m - Сменить модель
cc - Очистить контекст
p - Задать инструкцию (системную)
cp - Сбросить инструкцию

Ещё проекты: @just_openbots (тут много интересного)
{telebot.formatting.hlink("Исходный код","https://gitea.gulyaipole.fun/justuser/just_minigpt")}
""", parse_mode = "HTML")
	bot.send_message(message.chat.id, f"""Также настоятельно рекомендую подписаться на канал бота: @justuser31

Обратная связь ( @just_anonchat_bot ) : {telebot.formatting.hcode(":justuser")}""", parse_mode = "HTML")


### MAIN ###
from api import *

import traceback
import logging

setted_models = {}
system_prompts = {}
onoff = {}

@bot.message_handler(commands=['info','i'])
def info(message):
	global setted_models, system_prompts
	id = str(message.chat.id)
	if id not in setted_models:
		setted_models[id] = "0.1"
	if id not in system_prompts:
		prompt = "None"
	else:
		prompt = system_prompts[str(message.chat.id)]

	bot.send_message(message.chat.id, f"""____ Информация ____
Версия: {setted_models[id]}
System-prompt: {telebot.formatting.hcode(prompt)}
""", parse_mode="HTML")


############ MODEL SELECTION ############

@bot.message_handler(commands=['m'])
def set_model(message):
	bot.send_message(message.chat.id, "Выберите новую модель:", reply_markup=gen_markup(str(message.chat.id)))

def gen_markup(id):
	markup = InlineKeyboardMarkup()
	markup.row_width = 3
	markup.add(InlineKeyboardButton("0.1", callback_data=id+"_0.1"),
						InlineKeyboardButton("0.2", callback_data=id+"_0.2"),
						InlineKeyboardButton("3.5", callback_data=id+"_3.5") )
	return markup

@bot.callback_query_handler(func=lambda call: True)
def callback_query(call):
	global setted_models, iddb
	id = call.data.split("_")[0] ; m = call.data.split("_")[1]

	try:
		iddb.pop(id)
	except:
		pass

	if m == "0.1":
		setted_models[id] = "0.1"
	elif m == "0.2":
		setted_models[id] = "0.2"
	elif m == "3.5":
		setted_models[id] = "3.5"
	bot.send_message(int(id), "Успешно установлена новая модель 🤖")

##########################################

@bot.message_handler(commands=['p'])
def set_prompt(message):
	global system_prompts
	system_prompts[str(message.chat.id)] = message.text[3:]
	bot.reply_to(message, "Установлен новый system-prompt 🤖")
@bot.message_handler(commands=['cprompt','cp'])
def clear_prompt(message):
	global system_prompts
	try:
		system_prompts.pop(str(message.chat.id))
	except:
		pass
	bot.reply_to(message, "System-prompt очищен 🤖")


@bot.message_handler(commands=['cc'])
def clear_context(message):
	global iddb, history
	try:
	  iddb.pop(str(message.chat.id))
	except:
		pass
	try:
		history[id] = []
	except:
		pass
	bot.reply_to(message, "Контекст очищен 🤖")


@bot.message_handler(func=lambda message: True)
def echo_message(message):
	if bot.get_chat(message.chat.id).type == "private" or message.text[:2] == "/a":
		global setted_models, system_prompts

		if message.text[:2] == "/a":
			text = message.text[3:]
		else:
			text = message.text

		id = str(message.chat.id)
		if id not in setted_models:
			setted_models[id] = "0.1"

		if id in system_prompts:
			if setted_models[id] != 3.5:
				prompt = '[INST]' + system_prompts[id] + '[/INST]\n\n' + text
		else:
			prompt = text

		st = bot.send_message(message.chat.id, "Печатает...")
		try:
			predicted = gen(prompt, message.chat.id, setted_models[id])
		except:
			bot.send_message(message.chat.id, "Извините, возникла непредвиденная ошибка")

		try:
			bot.reply_to(message, predicted, parse_mode="Markdown")
		except Exception as e:
			bot.reply_to(message, predicted, parse_mode="HTML")
			logging.error(traceback.format_exc())
			print(predicted)

		bot.delete_message(message.chat.id, st.id)


############

bot.infinity_polling()
